import os
import select
import subprocess
import threading

from user_input_android import UserInputAndroid
from data_handler import DataHandler


class AndroidCommandExecutor:

    def __init__(self, path, channel):
        self.fifo_path = path
        self.android_input = UserInputAndroid(device_serial_id=channel)
        pass

    def execute_command(self, command, channel, input_file_path, env_vars=None):
        self.fifo_path = input_file_path
        try:
            print(f"run start {input_file_path}")
            #
            if os.path.exists(input_file_path):
                print("Exist file and removed")
                os.remove(input_file_path)

            result = subprocess.run(['mkfifo', input_file_path])
            if result.returncode != 0:
                print(f'Error creating FIFO: {result.stderr}')
                return False
            else:
                print('FIFO created successfully')

            print("Start execute command")
            os.chmod(input_file_path, 0o666)

            # Start a thread to read from the named pipe
            reader_thread = threading.Thread(target=self._read_fifo_end)
            reader_thread.start()

            # Run the command with output redirected to the named pipe and environment variables
            with open(input_file_path, 'wb') as fifo:
                print("subprocess start")
                process = subprocess.Popen(command, env=env_vars, shell=True, stdin=subprocess.PIPE, stdout=fifo)
                print(f"Return Code: {process.pid}")

                # Store kvs aws device process id
                db = DataHandler()
                data_object = db.get_device_process_data(device_id=channel)
                stream_data = {"device_serial_id": data_object[0]["device_serial_id"], "aws_pid_id": process.pid}
                db.update_process_data(unique_id=data_object[0]['id'], process_data=stream_data)
                print("subprocess end")

            # Wait for the command to complete or timeout
            process.wait()
            print("process wait")
            # Return the return code
            return process.returncode
        except subprocess.TimeoutExpired:
            # Handle timeout exception
            print("Command execution timed out.")
            return None
        except Exception as e:
            # Handle any other exceptions that may occur during command execution
            print(f"Error executing command: {e}")
            # error_message = e.args[0] if e.args else str(e)
            print("Error arguments :", e.args[0], e.args[1])
            return None

    def _read_fifo_end(self):
        print("Device fifo read start")
        self.pipe_fd = os.open(self.fifo_path, os.O_RDONLY)
        try:
            while True:
                reads, write, execute = select.select([self.pipe_fd], [], [], 1.0)
                if 0 < len(reads):
                    d = os.read(reads[0], 1024)
                    if d:
                        line = d.decode("utf-8", 'ignore').strip()
                        # Change with Android
                        self.android_input.handle_data(fifo_line=line)
                    else:
                        break
                else:
                    # Change with Android
                    self.android_input.fifo_ended()
        except KeyboardInterrupt:
            # Handle the Ctrl-C signal
            print("Pipeline interrupted by user")
        finally:
            os.close(self.pipe_fd)

