import os
import pathlib
import subprocess
from time import sleep
from ios_stream_master import StreamMaster
from device_type import Device_Type
from config_parser import Config
from data_handler import DataHandler


class MasterHandler:

    def __init__(self):
        print("---- init --- Master Handler init called")
        command = "source ~/.zshrc"
        subprocess.run(command, shell=True)
        pass

    def handle_fifo(self, fifo_line):
        print(f"Read line {fifo_line}")
        start_index = fifo_line.find("###")  # find the first occurrence of '###'
        if start_index != -1:
            start_index += 3  # skip over the leading '###'
            end_index = fifo_line.find("###", start_index)  # find the second occurrence of '###'
            if end_index != -1:
                substring = fifo_line[start_index:end_index]
                print(substring)  # prints "Dynamic Start Data"
                self.start_stop_handler(substring)
        pass

    def start_stop_handler(self, substring):
        print("start stop enter")
        elements = substring.split(" ")
        print(elements)
        if len(elements) > 1:
            start_stop = elements[0]
            channel = elements[1]
            config = Config()
            device_type = config.get_device_type(channel=channel)
            print(f"Device Type {device_type}")
            # Check Channel existed in pyson
            db = DataHandler()
            # get channel object
            device_start = db.check_device_start(device_id=channel)
            if device_start:
                print("Device already started")
                db.remove_all_duplicates(device_id=channel)
                device_start = False
            else:
                print("Device Need to start")

            if device_type == Device_Type.IOS.value:
                print("iOS")
                if "Start" in substring and device_start == False:
                    print("Start iOS Stream")
                    self.stream_ios(channel=channel, device_type=device_type)
                elif "Stop" in substring:
                    print("Stop iOS Stream")
                    self.stop_device_channel(channel=channel, device_type=device_type)
            elif device_type == Device_Type.ANDROID.value:
                print("Android")
                if "Start" in substring and device_start == False:
                    print("Start Android Stream")
                    self.start_android(channel=channel, device_type=device_type)
                elif "Stop" in substring:
                    print("Stop Android Stream")
                    self.stop_device_channel(channel=channel, device_type=device_type)
            pass
        else:
            print("")
        pass

    def start_ios_idb_streaming(self, device_id=None):
        port = 12345
        cmd = f'/usr/local/bin/idb list-targets && idb video-stream --fps 30 --format h264 --compression-quality 1.0 --udid {device_id} | ffmpeg -i - -vcodec copy -tune zerolatency -b:v 900k -framerate 30 -codec:v h264 -f rawvideo udp://0.0.0.0:{port}'
        process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        pid = process.pid
        print("The PID of the start_ios_idb is:", pid)
        # updating the idb pid process data in local database
        db = DataHandler()
        # get channel object
        data_object = db.get_device_process_data(device_id=device_id)
        # update idb process id in  channel object
        stream_data = {"device_serial_id": data_object[0]["device_serial_id"], "idb": pid}
        db.update_process_data(unique_id=data_object[0]['id'], process_data=stream_data)
        # run wda project after 5seconds
        sleep(5)
        self.run_ios_wda_project(device_serial_id=device_id)
        pass

    def stop_device_channel(self, channel, device_type):
        # Stop Process by PID and killed
        # get channel object using data handler
        db = DataHandler()
        data_object = db.get_device_process_data(device_id=channel)

        if device_type == Device_Type.IOS.value:
            # kill pid process
            kill_idb_list_command = "kill -SIGKILL " + str(data_object[0]["pid_id"])
            self.execute_system_command(kill_idb_list_command)

            # Kill the Device channel process
            kill_aws_streaming_command = "kill -SIGKILL " + str(data_object[0]["aws_pid_id"])
            self.execute_system_command(kill_aws_streaming_command)

            # kill idb pid process
            kill_idb_streaming_command = "kill -SIGKILL " + str(data_object[0]["idb"])
            self.execute_system_command(kill_idb_streaming_command)

        elif device_type == Device_Type.ANDROID.value:
            # kill scrcpy pid process
            kill_idb_list_command = "kill -SIGKILL " + str(data_object[0]["scrcpy_pid_id"])
            self.execute_system_command(kill_idb_list_command)

            # Kill the Device channel process
            kill_idb_streaming_command = "kill -SIGKILL " + str(data_object[0]["aws_pid_id"])
            self.execute_system_command(kill_idb_streaming_command)

            # kill adb pid process
            kill_adb_streaming_command = "kill -SIGKILL " + str(data_object[0]["adb"])
            self.execute_system_command(kill_adb_streaming_command)

        # after killed delete object from data base
        db.delete_process_data(data_object[0]['id'])

        pass

    def stream_ios(self, channel, device_type):
        # stream start
        print("idb companion start")
        list_idb = '/usr/local/bin/idb list-targets'
        process_list = subprocess.Popen(list_idb, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print("The PID of the list idb is:", process_list.pid)

        # store streaming details to local database
        stream_data = {"device_serial_id": channel, "pid_id": process_list.pid, "device_type": device_type,
                       "aws_pid_id": "", "idb": "", "adb": ""}
        db = DataHandler()
        db.store_device_process_data(process_data=stream_data)

        sleep(5)
        print("stream start")
        self.start_ios_idb_streaming(device_id=channel)
        ios_stream = StreamMaster()
        ios_stream.start_master_stream(channel_name=channel)
        return

    def start_android(self, channel, device_type):
        print("Android Start")
        # scrcpymz -s {channel} -P --video-bit-rate 4M --max-fps=30
        android_stream = f'scrcpymz -s {channel} -P --video-bit-rate 4M --max-fps=30'
        process_android = subprocess.Popen(android_stream, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        print("android stream scrcpy process id :", process_android.pid)
        # store scrcpy details to local database
        stream_data = {"device_serial_id": channel, "scrcpy_pid_id": process_android.pid, "device_type": device_type,
                       "idb": "", "adb": "", "aws_pid_id": ""}
        db = DataHandler()
        db.store_device_process_data(process_data=stream_data)

        aws_stream = StreamMaster()
        aws_stream.start_master_scrcpy(channel_name=channel)

    # run wda ios project in device
    def run_ios_wda_project(self, device_serial_id):
        try:
            project_path = str(pathlib.Path.home()) + "/aws-samples/WebDriverAgent-master/WebDriverAgent.xcodeproj"
            print(project_path)
            print(device_serial_id)
            command = "xcodebuild \
                           test-without-building \
                           -project " + project_path + "\
                            -scheme WebDriverAgentRunner" + "\
                           -destination 'platform=iOS,id=" + device_serial_id + "'"
            self.execute_system_command(command=command)
        except Exception as e:
            print('Unable to run wda project on iOS Device')
            print(e)

    def execute_system_command(self, command):
        try:
            process = subprocess.Popen(command, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                       cwd=os.getenv('HOME'))
            stdout, stderr = process.communicate()
            return stdout.decode('utf-8')
        except Exception as e:
            print('Error In Executing System Command ' + str(e))
            return None
