import json
import os
import subprocess
from datetime import datetime
from json import JSONDecodeError

from config_parser import Config
from data_handler import DataHandler
from device_type import Device_Type


class UserInput:

    def __init__(self):
        pass

    def handle_click(self):
        pass

    def handle_tap(self):
        pass

    def handle_long_press(self):
        pass

    def handle_swipe(self):
        pass

    def handle_keyboard_input(self):
        pass

    def handle_shortcut(self, command=None):
        """
        handles the shortcuts such as: HOME, BACK, SCREENSHOT
        Args:
            command:
                Command is constant for well known shortcuts.

        Returns:
            None
        """
        pass

    def execute_system_command(self, command):
        try:
            process = subprocess.Popen(command, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                       cwd=os.getenv('HOME'))
            stdout, stderr = process.communicate()
            return stdout.decode('utf-8')
        except Exception as e:
            print('Error In Executing System Command ' + str(e))
            return None

    def prase_touch_name(self, arguments):
        """
        prasing  the argument to return the touch name and listObj which contains the x and y values

        """
        if len(arguments) > 3:
            arguments[-1] = json.loads(arguments[-1].replace("'", "\""))
        parts = arguments[1:]
        listObj = parts[0].split(", ")
        touch_name = listObj[0]
        return touch_name, listObj

    def get_coordinates(self, input_value):
        try:
            dictionary = json.loads(input_value)
            value_at_index = list(dictionary.values())
            start_x = value_at_index[2]
            start_y = value_at_index[3]
        except JSONDecodeError as e:
            print('Decoding JSON has failed', e)
        return start_x, start_y

    def get_current_time(self):
        time = datetime.now().strftime('%Y/%m/%d %H:%M:%S.%f')[:-3]
        return time

    def stop_device_channel(self, channel):

        config = Config()
        device_type = config.get_device_type(channel=channel)
        print(f"Device Type {device_type}")

        # get channel object using data handler
        db = DataHandler()
        data_object = db.get_device_process_data(device_id=channel)

        if device_type == Device_Type.IOS.value:
            # kill pid process
            kill_idb_list_command = "kill -SIGKILL " + str(data_object[0]["pid_id"])
            self.execute_system_command(kill_idb_list_command)

            # kill idb pid process
            kill_idb_streaming_command = "kill -SIGKILL " + str(data_object[0]["idb"])
            self.execute_system_command(kill_idb_streaming_command)

            # Kill the Device channel process
            kill_idb_streaming_command = "kill -SIGKILL " + str(data_object[0]["aws_pid_id"])
            self.execute_system_command(kill_idb_streaming_command)
        elif device_type == Device_Type.ANDROID.value:
            # kill scrcpy pid process
            kill_idb_list_command = "kill -SIGKILL " + str(data_object[0]["scrcpy_pid_id"])
            self.execute_system_command(kill_idb_list_command)

            # Kill the Device channel process
            kill_idb_streaming_command = "kill -SIGKILL " + str(data_object[0]["aws_pid_id"])
            self.execute_system_command(kill_idb_streaming_command)

            # kill adb pid process
            kill_adb_streaming_command = "kill -SIGKILL " + str(data_object[0]["adb"])
            self.execute_system_command(kill_adb_streaming_command)

        # after killed delete object from data base
        db.delete_process_data(data_object[0]['id'])
        pass

