import math
import os
import random
from user_input import UserInput


class UserInputAndroid(UserInput):

    def __init__(self, device_serial_id):
        self.device_serial_id = ""
        self.x1 = 0
        self.y1 = 0
        self.x2 = 0
        self.y2 = 0
        if self.device_serial_id == "":
            self.device_serial_id = device_serial_id

    def handle_click(self):
        pass

    def handle_tap(self):
        print(f"adb tap action detected {str(math.trunc(self.x1))} {str(math.trunc(self.y1))}")
        command = "adb -s " + self.device_serial_id + " shell input tap " + str(math.trunc(self.x1)) + " " + str(math.trunc(self.y1))
        print(command)
        self.execute_system_command(command)
        pass

    def handle_long_press(self):
        # adb shell input  keyevent - - longpress
        pass

    def handle_swipe(self):
        print("swipe action detected")
        command = "adb -s " + self.device_serial_id + " shell input touchscreen swipe " + str(
            self.x1) + " " + str(self.y1) + " " + str(self.x2) + " " + str(self.y2)
        print(command)
        self.execute_system_command(command)
        pass

    def handle_home_press_action(self):
        command = "adb -s " + self.device_serial_id + " shell input keyevent KEYCODE_HOME"
        print(command)
        self.execute_system_command(command)
        pass

    def handle_keyboard_input(self, keyboard_value=None):
        command = "adb -s " + self.device_serial_id + " shell input text " + keyboard_value
        print(command)
        self.execute_system_command(command)
        pass

    def handle_shortcut(self, command=None):
        pass

    def handle_app_install(self):
        file_path = os.getenv('HOME') + "/Downloads/5gmark_pro.apk"
        print("adb -s " + self.device_serial_id + " install -r -t '" + file_path + "'")
        android_install_command = "adb -s " + self.device_serial_id + " install " + file_path
        print(android_install_command)
        self.execute_system_command(android_install_command)
        pass

    def handle_screenshot(self):
        print("screen shot detected")
        # take screenshot and save in sdcard
        random_value = random.randint(0, 10000000)
        print("adb -s " + self.device_serial_id + f" shell screencap -p /sdcard/{random_value}.png")
        command = "adb -s " + self.device_serial_id + f" shell screencap -p /sdcard/{random_value}.png"
        print(command)
        self.execute_system_command(command)

        # pull  screenshot from  sdcard to your local
        path = os.getenv('HOME')
        command_pull = "adb -s " + self.device_serial_id + f" pull /sdcard/{random_value}.png " + path
        print(command_pull)
        self.execute_system_command(command_pull)

        # delete  screenshot from  sdcard
        command_delete = "adb -s " + self.device_serial_id + f" shell rm /sdcard/{random_value}.png"
        print(command_delete)
        self.execute_system_command(command_delete)

    def launch_scrcpy(self):
        print("launch_scrcpy")
        print(self.device_serial_id)
        command = "scrcpy -s " + self.device_serial_id
        print(command)
        self.execute_system_command(command)

    # Handle gestures triggered from aws streaming and pass to related functions
    def handle_adb_gesture_element(self, arguments):
        touch_name = arguments[0]
        print(touch_name)
        if touch_name == 'input.gestureStart':
            # Consider it first time get gesture start
            self.x1 = 0
            self.y1 = 0
            self.x2 = 0
            self.y2 = 0
            print("Gesture start")
        elif touch_name == 'input.keyUp' or touch_name == 'input.keyDown':
            self.handle_keyboard_input()
        elif touch_name == 'input.touchDown':
            print("testing")
            strObj = arguments[2]
            self.x1, self.y1 = self.get_coordinates(input_value=strObj)
        elif touch_name == 'input.touchUp':
            # Touch up only trigger when x2 and y2 zero (it means swipe detected)
            if self.x1 != 0 and self.y1 != 0 and self.x2 == 0 and self.y2 == 0:
                self.handle_tap()
        elif touch_name == 'input.touchMove':
            strObj = arguments[2]
            if self.x1 == 0 and self.y1 == 0:
                self.x1, self.y1 = self.get_coordinates(input_value=strObj)
            self.x2, self.y2 = self.get_coordinates(input_value=strObj)
            print(f"Values Touch move {self.x1} {self.y1} {self.x2} {self.y2} {strObj}")
        elif touch_name == 'input.gestureStop':
            print("Gesture stop")
            if self.x1 != 0 and self.y1 != 0 and self.x2 != 0 and self.y2 != 0:
                self.handle_swipe()
        elif touch_name == 'screen.capture':
            print("screen shot detected")
            self.handle_screenshot()
        elif touch_name == 'app.install':
            print("app install detected")
            self.handle_app_install()
        elif touch_name == "stop.device":
            print("Stop Device called")
            channel = arguments[1]
            self.stop_device_channel(channel=channel)

    def handle_data(self, fifo_line):
        print(f"from android master {fifo_line} --- END --- \n")
        start_index = fifo_line.find("###")  # find the first occurrence of '###'
        if start_index != -1:
            start_index += 3  # skip over the leading '###'
            end_index = fifo_line.find("###", start_index)  # find the second occurrence of '###'
            if end_index != -1:
                substring = fifo_line[start_index:end_index]
                listObj = substring.split(", ")
                if len(listObj) > 2:
                    self.handle_adb_gesture_element(arguments=listObj)
                else:
                    print("data is not proper")
        pass

    # Handle if fifo ended check x1 y1 y2 x2 not equal to zero and trigger to swipe func
    def fifo_ended(self):
        self.x1 = 0
        self.y1 = 0
        self.x2 = 0
        self.y2 = 0
        if self.x1 != 0 and self.y1 != 0 and self.x2 != 0 and self.y2 != 0:
            self.handle_swipe()
