import wda
import math
import os
import random
from user_input import UserInput


class UserInputIOS(UserInput):
    usb_client = None

    def __init__(self, device_serial_id, port):
        self.date_string = self.get_current_time()
        self.device_serial_id = device_serial_id
        self.x1 = 0
        self.x2 = 0
        self.y1 = 0
        self.y2 = 0
        if self.usb_client is None:
            self.usb_client = wda.USBClient(udid=device_serial_id, port=port)

    def handle_click(self):
        print("click action triggered")
        self.usb_client.click(x=self.x1, y=self.y1)
        pass

    def handle_tap(self):
        print(f"tap action triggered with values  {self.x1} {self.y1} ")
        self.usb_client.tap(x=self.x1, y=self.y1)
        pass

    def handle_long_press(self):
        print("long press action triggered")
        self.usb_client.tap_hold(x=self.x1, y=self.y1, duration=1.0)
        pass

    def handle_swipe(self):
        print(f"swipe action triggered with values  {self.x1} {self.y1} {self.x2} {self.y2}")
        # local unit testing working with after converting float values
        self.usb_client.swipe(math.floor(self.x1), math.floor(self.y1), math.floor(self.x2), math.floor(self.y2))
        # local unit testing working with below int values
        # self.usb_client.swipe(self.x1, self.y1, self.x2, self.y2)
        print(f"after swipe triggered with values {self.x1} {self.y1} {self.x2} {self.y2}")
        self.x1 = 0
        self.y1 = 0
        self.x2 = 0
        self.y2 = 0
        pass

    def get_ios_screen_size(self):
        size = self.usb_client.window_size()
        height = size.height
        width = size.width
        return [height, width]

    def handle_keyboard_input(self, keyboard_value=None):
        self.usb_client.send_keys(value=keyboard_value)
        pass

    def handle_home_press_action(self):
        self.usb_client.press("home")
        pass

    def handle_keyboard_dismiss(self):
        """
        keyboard not working for now

        """
        print("handle_keyboard_dismiss")
        self.usb_client.keyboard_dismiss()
        pass

    def handle_screenshot(self):
        print("")
        path = os.getenv('HOME')
        # save screenshot with random generated number
        self.usb_client.screenshot().save(path + f"/{random.randint(0, 10000000)}.png")

    # install ipa file using i device installer command
    def handle_app_installation(self):
        print("handle_app_installation")
        path = os.getenv('HOME') + "/Downloads/27-03-S.ipa"
        install_app_cmd = "ideviceinstaller -u " + self.device_serial_id + " --install " + path
        self.execute_system_command(command=install_app_cmd)
        pass

    # Handle gestures triggered from aws streaming and pass to related functions
    def handle_gesture_element(self, arguments):
        touch_name = arguments[0]
        print(touch_name)
        if touch_name == 'input.gestureStart':
            # Consider it first time get gesture start
            print("Gesture start")
        elif touch_name == 'input.keyUp' or touch_name == 'input.keyDown':
            self.handle_home_press_action()
        elif touch_name == 'input.touchDown':
            strObj = arguments[2]
            self.x1, self.y1 = self.get_coordinates(input_value=strObj)
        elif touch_name == 'input.touchUp':
            # Touch up only trigger when x2 and y2 zero (it means swipe detected)
            if self.x1 != 0 and self.y1 != 0 and self.x2 == 0 and self.y2 == 0:
                self.handle_tap()
        elif touch_name == 'input.touchMove':
            strObj = arguments[2]
            if self.x1 == 0 and self.y1 == 0:
                self.x1, self.y1 = self.get_coordinates(input_value=strObj)
            self.x2, self.y2 = self.get_coordinates(input_value=strObj)
            print(f"Values Touch move {self.x1} {self.y1} {self.x2} {self.y2} {strObj}")
        elif touch_name == 'input.gestureStop':
            print("Gesture stop")
            if self.x1 != 0 and self.y1 != 0 and self.x2 != 0 and self.y2 != 0:
                self.handle_swipe()
        elif touch_name == 'screen.capture':
            print("screen shot detected")
            self.handle_screenshot()
        elif touch_name == 'app.install':
            print("app install detected")
            self.handle_app_installation()
        elif touch_name == "stop.device":
            print("Stop Device called")
            channel = arguments[1]
            self.stop_device_channel(channel=channel)

    def handle_data(self, fifo_line):
        start_index = fifo_line.find("###")  # find the first occurrence of '###'
        if start_index != -1:
            start_index += 3  # skip over the leading '###'
            end_index = fifo_line.find("###", start_index)  # find the second occurrence of '###'
            if end_index != -1:
                substring = fifo_line[start_index:end_index]
                listObj = substring.split(", ")
                if len(listObj) > 2:
                    self.handle_gesture_element(arguments=listObj)
                else:
                    print("data is not proper")
        pass

    # Handle if fifo ended check x1 y1 y2 x2 not equal to zero and trigger to swipe func
    def fifo_ended(self):
        if self.x1 != 0 and self.y1 != 0 and self.x2 != 0 and self.y2 != 0:
            self.handle_swipe()

